@@ -1,6 +1,8 @@
 import numpy as np
 from typing import List, Optional, Union, Dict, Any
-from .compat import pd_DataFrame, pd_Series
+from .compat import pd_DataFrame, pd_Series, pd_read_parquet
+import blosc
+import dill
 
 MeanMatchType = Union[int, float, Dict[str, float], Dict[str, int]]
 VarSchemType = Optional[Union[List[str], Dict[str, List[str]]]]
@@ -108,6 +110,23 @@ def stratified_categorical_folds(y, nfold):
         yield (np.setdiff1d(range(elements), v), v)
 
 
+# https://stackoverflow.com/questions/664014/what-integer-hash-function-are-good-that-accepts-an-integer-hash-key
+# We don't really need to worry that much about diffusion
+# since we take % n at the end, and n (mmc) is usually
+# very small. This hash performs well enough in testing.
+def hash_int32(x):
+    """
+    A hash function which generates random uniform (enough)
+    int32 integers. Used in mean matching and initialization.
+    """
+    assert isinstance(x, np.ndarray)
+    assert x.dtype == "int32", "x must be int32"
+    x = ((x >> 16) ^ x) * 0x45D9F3B
+    x = ((x >> 16) ^ x) * 0x45D9F3B
+    x = (x >> 16) ^ x
+    return x
+
+
 def ensure_rng(
     random_state: Optional[Union[int, np.random.RandomState]] = None
 ) -> np.random.RandomState:
@@ -125,6 +144,22 @@ def ensure_rng(
     return random_state
 
 
+def load_kernel(filepath, n_threads=None):
+    n_threads = blosc.detect_number_of_cores() if n_threads is None else n_threads
+    blosc.set_nthreads(n_threads)
+    with open(filepath, "rb") as f:
+        kernel = dill.loads(blosc.decompress(dill.load(f)))
+
+    if kernel.original_data_class == "pd_DataFrame":
+        kernel.working_data = pd_read_parquet(kernel.working_data)
+        for col in kernel.working_data.columns:
+            kernel.working_data[col] = kernel.working_data[col].astype(
+                kernel.working_dtypes[col]
+            )
+
+    return kernel
+
+
 def _get_missing_stats(data: np.ndarray):
     """
     This function is seperate because this data is needed
